/*
 * Copyright (c) 1997-2006 Erez Zadok
 * Copyright (c) 1997-2006 Nikolai Joukov
 * Copyright (c) 2001-2006 Stony Brook University
 * Copyright (c) 1997-2000 Columbia University
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 *  $Id: print.c,v 1.17 2006/12/13 17:00:21 kolya Exp $
 */

/* Print debugging functions */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */
#include "fist.h"
#include "raif.h"


static int fist_debug_var = 0;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#define PageUptodate(page) Page_Uptodate(page)
#endif


/* get value of debugging variable */
int fist_get_debug_value(void)
{
	return fist_debug_var;
}

/* set debug level variable and return the previous value */
int fist_set_debug_value(int val)
{
	int prev = fist_debug_var;

	printk("raif: setting debug level to %d\n", val);
	fist_debug_var = val;
	return prev;
}

/*
 * Utilities used by both client and server
 * Standard levels:
 * 0) no debugging
 * 1) hard failures
 * 2) soft failures
 * 3) current test software
 * 4) main procedure entry points
 * 5) main procedure exit points
 * 6) utility procedure entry points
 * 7) utility procedure exit points
 * 8) obscure procedure entry points
 * 9) obscure procedure exit points
 * 10) random stuff
 * 11) all <= 1
 * 12) all <= 2
 * 13) all <= 3
 * ...
 */

static char buf[4096];

void fist_dprint_internal(const char *file, const char *function, int line, int level, char *str,...)
{
	va_list ap;
	int var = fist_get_debug_value();

	if (level >= 10 || level < 0) {
		printk("<0>fist_dprint_internal: Invalid level passed from %s:%s:%d\n", file, function, line);
	}  

	if (var == level || (var > 10 && (var - 10) >= level)) {
		va_start(ap, str);
		vsnprintf(buf, 4096, str, ap);
		printk("%s", buf);
		va_end(ap);
	}
	return;
}


static int num_indents = 0;
char indent_buf[80] = "                                                                               ";
char * add_indent(void)
{
	indent_buf[num_indents] = ' ';
	num_indents++;
	if (num_indents > 79)
		num_indents = 79;
	indent_buf[num_indents] = '\0';
	return indent_buf;
}

char * del_indent(void)
{
	if (num_indents <= 0)
		return "<IBUG>";
	indent_buf[num_indents] = ' ';
	num_indents--;
	indent_buf[num_indents] = '\0';
	return indent_buf;
}

void fist_print_page_bytes(char *str, page_t *page)
{
#ifndef FIST_NO_DPRINT
	int i;
	char *cp = (char *) page_address(page);

	fist_dprint(8, "PPB %s (0x%x):", str, (int) page);

	for (i=0;i<10;i++)
		fist_dprint(8, "%c,",cp[i]);
	fist_dprint(8, "\n");
#endif
}


void fist_print_buffer_flags(char *str, struct buffer_head *buffer)
{
# define buffer_protected(buffer) (-1)
	if (!buffer) {
		printk("PBF %s 0x%x\n", str, (int) buffer);
		return;
	}

	fist_dprint(8, "PBF %s 0x%x: Uptodate:%d Dirty:%d Locked:%d Req:%d Protected:%d\n",
			str,
			(int) buffer,
			buffer_uptodate(buffer),
			buffer_dirty(buffer),
			buffer_locked(buffer),
			buffer_req(buffer),
			buffer_protected(buffer)
		   );
}


void fist_print_page_flags(char *str, page_t *page)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10)
	/*
	 * XXX: 2.4.10 forgot to export the ksym to swapper_space,
	 * which is used in PageSwapCache().
	 */
# define fist_PageSwapCache(page) (-1)
#else
# define fist_PageSwapCache(page) (PageSwapCache(page) ? 1 : 0)
#endif /* kernel 2.4.10 and newer */

#ifdef PageDecrAfter
#define fist_PageDecrAfter(page)	(PageDecrAfter(page) ? 1 : 0)
#else  /* not PageDecrAfter */
#define fist_PageDecrAfter(page)	(-1)
#endif /* not PageDecrAfter */

	fist_dprint(8, "PPF %s 0x%x/0x%x: Locked:%d Error:%d Referenced:%d Uptodate:%d DecrAfter:%d Slab:%d SwapCache:%d Reserved:%d\n",
			str,
			(int) page,
			(int) page->index,
			(PageLocked(page) ? 1 : 0),
			(PageError(page) ? 1 : 0),
			(PageReferenced(page) ? 1 : 0),
			(PageUptodate(page) ? 1 : 0),
			fist_PageDecrAfter(page),
			(PageSlab(page) ? 1 : 0),
			fist_PageSwapCache(page),
			(PageReserved(page) ? 1 : 0)
		   );
}


void fist_print_inode(char *str, const inode_t *inode)
{
	inode_t *hidden_inode;

	if (!inode) {
		printk("PI:%s: NULL INODE PASSED!\n", str);
		return;
	}
	fist_dprint(8, "\n");
	fist_dprint(8, "=====Inode Data=====\n");
	fist_dprint(8, "PI:%s: %s=%lu\n", str, "i_ino", inode->i_ino);
	fist_dprint(8, "PI:%s: %s=%u\n", str, "i_count", atomic_read(&inode->i_count));
	fist_dprint(8, "PI:%s: %s=%u\n", str, "i_nlink", inode->i_nlink);
	fist_dprint(8, "PI:%s: %s=%x\n", str, "i_mode", inode->i_mode);
	if (S_ISDIR(inode->i_mode)) 
		fist_dprint(8, "inode is a directory!\n");
	else 
		fist_dprint(8, "inode is a file!\n");	
	fist_dprint(8, "PI:%s: %s=%x\n", str, "i_size", inode->i_size);
	fist_dprint(8, "PI:%s: %s=%p\n", str, "i_op", inode->i_op);
	fist_dprint(8, "PI:%s: %s=%x\n", str, "i_sb", (int) inode->i_sb);

	if(itopd(inode)) {
		PASSERT(itopd(inode));
		fist_dprint(8, "ibnum=%d\n", ibnum(inode));
#ifdef RAIF_TAG
		fist_dprint(8, "level=%d, sush=%d\n", LEVEL(inode), STRIPING_UNIT_SHIFT(inode));
#endif
	}
	fist_dprint(8, "\n");   /* printing underlying inodes, if any */

	if(itopd(inode)) {
		int bindex;

		fist_dprint(8, "Hidden Inodes\n");
		if(ibnum(inode) <= 0)
			fist_dprint(8, "NONE\n");
		else
			for (bindex = 0; bindex < ibnum(inode); bindex++) {
				hidden_inode = itohi_index(inode, bindex);
				fist_dprint(8, "---------------------\n");

				if(!hidden_inode) {
					fist_dprint(8, "Hidden Inode # %d = NULL", bindex);
					continue;
				}

				fist_dprint(8, "Hidden Inode # %d: %s: %s=%lu\n", bindex, str, "i_ino", hidden_inode->i_ino);
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%u\n", bindex, str, "i_count", atomic_read(&hidden_inode->i_count));
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%u\n", bindex, str, "i_nlink", hidden_inode->i_nlink);
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%x\n", bindex, str, "i_mode", hidden_inode->i_mode);
				if (S_ISDIR(inode->i_mode)) 
					fist_dprint(8, "inode is a directory!\n");
				else 
					fist_dprint(8, "inode is a file!\n");	
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%x\n", bindex, str, "i_size", hidden_inode->i_size);
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%p\n", bindex, str, "i_op", hidden_inode->i_op);
				fist_dprint(8, "Hidden Inode # %d: %s: %s=%x\n", bindex, str, "i_sb", (int) hidden_inode->i_sb);
			}
	}
}


void fist_print_pte_flags(char *str, const page_t *page)
{
	unsigned long address;

	PASSERT(page);
	address = page->index;
	fist_dprint(8, "PTE-FL:%s index=0x%x\n", str, (int) address);
}


void fist_print_vma(char *str, const vm_area_t *vma)
{
	return;
}


void fist_print_file(char *str, const file_t *file)
{
	file_t *hidden_file;

	if (!file) {
		fist_dprint(8, "PF:%s: NULL FILE PASSED!\n", str);
		return;
	}
	fist_dprint(8, "=====File Data=====\n");

	fist_dprint(8, "PF:%s: %s=0x%p\n", str, "f_dentry", file->f_dentry);
	fist_dprint(8, "PF:%s: file's dentry name=%s\n", str, file->f_dentry->d_name);
	if (file->f_dentry->d_inode) {
		if (S_ISDIR(file->f_dentry->d_inode->i_mode)) 
			fist_dprint(8, "dentry is a directory!\n");
		else 
			fist_dprint(8, "dentry is a file!\n");
	}
	fist_dprint(8, "PF:%s: %s=0x%p\n", str, "f_op", file->f_op);
	fist_dprint(8, "PF:%s: %s=0x%x\n", str, "f_mode", file->f_mode);
	fist_dprint(8, "PF:%s: %s=0x%x\n", str, "f_pos", file->f_pos);
	fist_dprint(8, "PF:%s: %s=%u\n", str, "f_count", file->f_count);
	fist_dprint(8, "PF:%s: %s=0x%x\n", str, "f_flags", file->f_flags);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	fist_dprint(8, "PF:%s: %s=0x%x\n", str, "f_reada", file->f_reada);
	fist_dprint(8, "PF:%s: %s=%d\n", str, "f_ramax", file->f_ramax);
	fist_dprint(8, "PF:%s: %s=%d\n", str, "f_raend", file->f_raend);
	fist_dprint(8, "PF:%s: %s=%d\n", str, "f_ralen", file->f_ralen);
	fist_dprint(8, "PF:%s: %s=%d\n", str, "f_rawin", file->f_rawin);
#endif

	fist_dprint(8, "PF:%s: %s=%lu\n", str, "f_version", file->f_version);
	if(ftopd(file))
		fist_dprint(8, "fbnum=%d\n", fbnum(file));
	fist_dprint(8, "\n"); 

	if(ftopd(file)) {
		int bindex;
		fist_dprint(8, "Hidden Files: \n");
		for (bindex = 0; bindex < fbnum(file); bindex++) {

			hidden_file = ftohf_index(file, bindex);
			fist_dprint(8, "---------------------\n");
			if(!hidden_file) {
				fist_dprint(8, "Hidden File # %d = NULL\n", bindex);
				continue;
			}

			fist_dprint(8, "Hidden File # %d :%s: %s=0x%p\n", bindex, str, "f_dentry", hidden_file->f_dentry);
			fist_dprint(8, "Hidden File # %d :%s: dentry's name=%s\n", bindex, str, hidden_file->f_dentry->d_name);
			if (hidden_file->f_dentry->d_inode) {
				if (S_ISDIR(hidden_file->f_dentry->d_inode->i_mode)) 
					fist_dprint(8, "dentry is a directory!\n");
				else 
					fist_dprint(8, "dentry is a file!\n");	
			}
			fist_dprint(8, "Hidden File # %d :%s: %s=0x%p\n", bindex, str, "f_op", hidden_file->f_op);
			fist_dprint(8, "Hidden File # %d :%s: %s=0x%x\n", bindex, str, "f_mode", hidden_file->f_mode);
			fist_dprint(8, "Hidden File # %d :%s: %s=0x%x\n", bindex, str, "f_pos", hidden_file->f_pos);
			fist_dprint(8, "Hidden File # %d :%s: %s=%u\n", bindex, str, "f_count", hidden_file->f_count);
			fist_dprint(8, "Hidden File # %d :%s: %s=0x%x\n", bindex, str, "f_flags", hidden_file->f_flags);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
			fist_dprint(8, "Hidden File # %d :%s: %s=0x%x\n", bindex, str, "f_reada", hidden_file->f_reada);
			fist_dprint(8, "Hidden File # %d :%s: %s=%d\n", bindex, str, "f_ramax", hidden_file->f_ramax);
			fist_dprint(8, "Hidden File # %d :%s: %s=%d\n", bindex, str, "f_raend", hidden_file->f_raend);
			fist_dprint(8, "Hidden File # %d :%s: %s=%d\n", bindex, str, "f_ralen", hidden_file->f_ralen);
			fist_dprint(8, "Hidden File # %d :%s: %s=%d\n", bindex, str, "f_rawin", hidden_file->f_rawin);
#endif
			fist_dprint(8, "Hidden File # %d :%s: %s=%lu\n", bindex, str, "f_version", hidden_file->f_version);
		}
	}
}


void fist_print_dentry(char *str, const dentry_t *dentry)
{
	dentry_t *hidden_dentry = NULL;

	if (!dentry) {
		fist_dprint(8, "PD:%s: NULL DENTRY PASSED!\n", str);
		return;
	}
	if (IS_ERR(dentry)) {
		fist_dprint(8, "PD:%s: ERROR DENTRY (%ld)!\n", str, PTR_ERR(dentry));
		return;
	}

	fist_dprint(8, "=====Dentry Data=====\n");

	fist_dprint(8, "PD:%s: %s=%d\n", str, "d_count",
			atomic_read(&dentry->d_count));
	fist_dprint(8, "PD:%s: %s=%x\n", str, "d_flags", (int) dentry->d_flags);
	fist_dprint(8, "PD:%s: %s=%p\n", str, "d_inode",
			dentry->d_inode);
	fist_dprint(8, "PD:%s: %s=%p\n", str, "d_parent",
			dentry->d_parent);
	fist_dprint(8, "PD:%s: %s=\"%s\"\n", str, "d_parent->d_name.name", dentry->d_parent->d_name.name);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_mounts", (int) dentry->d_mounts);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_covers", (int) dentry->d_covers);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_hash", (int) & dentry->d_hash);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_lru", (int) & dentry->d_lru);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_child", (int) & dentry->d_child);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_subdirs", (int) & dentry->d_subdirs);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_alias", (int) & dentry->d_alias);
	//    fist_dprint(8, "PD:%s: %s=%x\n", str, "d_name", (int) & dentry->d_name);
	//
	fist_dprint(8, "PD:%s: %s=\"%s\"\n", str, "d_name.name", dentry->d_name.name);
	if (dentry->d_inode) {
		char t;
		if (S_ISDIR(dentry->d_inode->i_mode))
			t = 'd';
		else if (S_ISLNK(dentry->d_inode->i_mode))
			t = 'l';
		else if (S_ISCHR(dentry->d_inode->i_mode))
			t = 'c';
		else if (S_ISBLK(dentry->d_inode->i_mode))
			t = 'b';
		else if (S_ISREG(dentry->d_inode->i_mode))
			t = 'f';
		else 
			t = '?';
		fist_dprint(8, "PD:%s: dentry->d_inode->i_mode: %c%o\n", str, t, dentry->d_inode->i_mode);
		fist_dprint(8, "PD:%s: dentry->d_inode->i_size: %d\n", str, dentry->d_inode->i_size);
	}
	fist_dprint(8, "PD:%s: %s=%d\n", str, "d_name.len", (int) dentry->d_name.len);
	fist_dprint(8, "PD:%s: %s=%p\n", str, "d_op", dentry->d_op);
	fist_dprint(8, "PD:%s: %s=%p\n", str, "d_sb");
	fist_dprint(8, "PD:%s: %s=%x\n", str, "d_fsdata", (int) dentry->d_fsdata);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	fist_dprint(8, "PD:%s:%s=%d\n", str, "list_empty(d_hash)", list_empty(&((dentry_t *)dentry)->d_hash));
#else
	fist_dprint(8, "PD:%s: %s=%d\n", str, "hlist_unhashed(d_hash)", hlist_unhashed(&((dentry_t *)dentry)->d_hash));
#endif
	if(dtopd(dentry)) {
		fist_dprint(8, "PD:%s: dbnum=%d\n", str, dbnum(dentry));
	}
	fist_dprint(8, "\n");

	if(dtopd(dentry)) {
		int bindex;

		PASSERT(dtopd(dentry));

		fist_dprint(8, "Hidden Dentries:\n");

		for (bindex = 0; bindex < dbnum(dentry); bindex++) {

			hidden_dentry = dtohd_index(dentry, bindex);
			fist_dprint(8, "---------------------\n");
			if(!hidden_dentry) {
				fist_dprint(8, "Hidden Dentry # %d = NULL\n", bindex);
				continue;
			}

			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%d\n", bindex, str, "d_count",
					atomic_read(&hidden_dentry->d_count));
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%x\n", bindex, str, "d_flags", (int) hidden_dentry->d_flags);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%x\n", bindex, str, "d_inode",
					(int) hidden_dentry->d_inode);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%x\n", bindex, str, "d_parent",
					(int) hidden_dentry->d_parent);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=\"%s\"\n", bindex, str, "d_name.name", hidden_dentry->d_name.name);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=\"%s\"\n", bindex, str, "d_parent->d_name.name", hidden_dentry->d_parent->d_name.name);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%d\n", bindex, str, "d_name.len", (int) hidden_dentry->d_name.len);
			if (hidden_dentry->d_inode) {
				char t;
				if (S_ISDIR(hidden_dentry->d_inode->i_mode))
					t = 'd';
				else if (S_ISLNK(hidden_dentry->d_inode->i_mode))
					t = 'l';
				else if (S_ISCHR(hidden_dentry->d_inode->i_mode))
					t = 'c';
				else if (S_ISBLK(hidden_dentry->d_inode->i_mode))
					t = 'b';
				else if (S_ISREG(hidden_dentry->d_inode->i_mode))
					t = '-';
				else 
					t = '?';
				fist_dprint(8, "Hidden Dentry # %d :%s: dentry->d_inode->i_mode: %c%o\n", bindex, str, t, hidden_dentry->d_inode->i_mode);
				fist_dprint(8, "Hidden Dentry # %d :%s: dentry->d_inode->i_size: %d\n", bindex, str, hidden_dentry->d_inode->i_size);
			}
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%p\n", bindex, str, "d_op", hidden_dentry->d_op);
			fist_dprint(8, "Hidden Dentry # %d :%s: %s=%x\n", bindex, str, "d_sb",
					(int) hidden_dentry->d_sb);
			fist_dprint(8, "Hidden Dentry # %d :%s=%x\n", bindex, str, "d_fsdata", (int) hidden_dentry->d_fsdata);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
			fist_dprint(8, "HD#%d:%s: %s=%d\n", bindex, str, "list_empty(d_hash)", list_empty(&((dentry_t *)hidden_dentry)->d_hash));
#else
			fist_dprint(8, "HD#%d:%s: %s=%d\n", bindex, str, "hlist_unhashed(d_hash)", hlist_unhashed(&((dentry_t *)hidden_dentry)->d_hash));
#endif
		}
	}
}



void fist_checkinode(inode_t *inode, char *msg)
{
	if (!inode) {
		printk(KERN_WARNING "fist_checkinode - inode is NULL! (%s)\n", msg);
		return;
	}
	if (!itopd(inode)) {
		fist_dprint(8, "fist_checkinode(%ld) - no private data (%s)\n", inode->i_ino, msg);
		return;
	}
	if (!itohi_index(inode, 0)) {
		fist_dprint(8, "fist_checkinode(%ld) - underlying is NULL! (%s)\n", inode->i_ino, msg);
		return;
	}
	if (!inode->i_sb) {
		fist_dprint(8, "fist_checkinode(%ld) - inode->i_sb is NULL! (%s)\n", inode->i_ino, msg);
		return;
	}
	fist_dprint(8, "inode->i_sb->s_type %p\n", inode->i_sb->s_type);
	if (!inode->i_sb->s_type) {
		fist_dprint(8, "fist_checkinode(%ld) - inode->i_sb->s_type is NULL! (%s)\n", inode->i_ino, msg);
		return;
	}
	fist_dprint(6, "CI: %s: inode->i_count = %d, hidden_inode->i_count = %d, inode = %d, sb = %s, hidden_sb = %s\n",
			msg, atomic_read(&inode->i_count),
			atomic_read(&itohi_index(inode, 0)->i_count), inode->i_ino,
			inode->i_sb->s_type->name, itohi_index(inode, 0)->i_sb->s_type->name);
}

void fist_print_sb(char *str, const super_block_t *sb)
{
	super_block_t *hidden_superblock;

	if (!sb) {
		fist_dprint(8, "PSB:%s: NULL SB PASSED!\n", str);
		return;
	}

	fist_dprint(8, "=====SuperBlock=====\n");

	fist_dprint(8, "PSB:%s: %s=%lu\n", str, "s_blocksize", (int) sb->s_blocksize);
	fist_dprint(8, "PSB:%s: %s=%u\n", str, "s_blocksize_bits", (int) sb->s_blocksize_bits);
	fist_dprint(8, "PSB:%s: %s=0x%x\n", str, "s_flags", (int) sb->s_flags);
	fist_dprint(8, "PSB:%s: %s=0x%x\n", str, "s_magic", (int) sb->s_magic);
	fist_dprint(8, "PSB:%s: %s=%llu\n", str, "s_maxbytes", sb->s_maxbytes);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,8)
	fist_dprint(8, "PSB:%s: %s=%d\n", str, "s_count", (int) sb->s_count);
	fist_dprint(8, "PSB:%s: %s=%d\n", str, "s_active", (int) atomic_read(&sb->s_active));
#endif /* kernels 2.4.8 and newer */
	if(stopd(sb))
		fist_dprint(8, "sbnum=%d\n", sbnum(sb));
	fist_dprint(8, "\n");

	if(stopd(sb)) {
		int global_bindex;
		fist_dprint(8, "Hidden SuperBlock: \n");
		for (global_bindex = 0; global_bindex < sbnum(sb); global_bindex++) {

			hidden_superblock = stohs_index(sb, global_bindex);
			fist_dprint(8, "--------------------\n");
			if(!hidden_superblock) {
				fist_dprint(8, "Hidden SB # %d = NULL", global_bindex);
				continue;
			}

			fist_dprint(8, "Hidden SB # %d:%s: %s=%lu\n", global_bindex, str, "s_blocksize", (int) hidden_superblock->s_blocksize);
			fist_dprint(8, "Hidden SB # %d:%s: %s=%u\n", global_bindex, str, "s_blocksize_bits", (int) hidden_superblock->s_blocksize_bits);
			fist_dprint(8, "Hidden SB # %d:%s: %s=0x%x\n", global_bindex, str, "s_flags", (int) hidden_superblock->s_flags);
			fist_dprint(8, "Hidden SB # %d:%s: %s=0x%x\n", global_bindex, str, "s_magic", (int) hidden_superblock->s_magic);
			fist_dprint(8, "Hidden SB # %d:%s: %s=%llu\n", global_bindex, str, "s_maxbytes", hidden_superblock->s_maxbytes);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,8)
			fist_dprint(8, "Hidden SB # %d:%s: %s=%d\n", global_bindex, str, "s_count", (int) hidden_superblock->s_count);
			fist_dprint(8, "Hidden SB # %d:%s: %s=%d\n", global_bindex, str, "s_active", (int) atomic_read(&hidden_superblock->s_active));
#endif /* kernels 2.4.8 and newer */

		}
	}
}

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
